@@ -25,7 +25,7 @@
 #ifndef __PANEL_COMPATIBILITY_H__
 #define __PANEL_COMPATIBILITY_H__
 
-#include <glib/gmacros.h>
+#include <glib.h>
 #include <gconf/gconf-client.h>
 
 G_BEGIN_DECLS
@@ -34,6 +34,12 @@ void panel_compatibility_migrate_panel_id_list (GConfClient *client);
 void panel_compatibility_maybe_copy_old_config (GConfClient *client);
 void panel_compatibility_migrate_applications_scheme (GConfClient *client,
 						      const char  *key);
+void panel_compatibility_migrate_screenshot_action (GConfClient *client,
+						   const char  *id);
+void panel_compatiblity_migrate_settings_menu_button (GConfClient *client,
+						      const char  *id);
+
+gchar *panel_compatibility_get_applet_iid (const gchar *id);
 
 G_END_DECLS
 
