@@ -325,6 +325,14 @@ panel_lockdown_get_locked_down (void)
         return panel_lockdown.locked_down;
 }
 
+gboolean
+panel_lockdown_get_not_locked_down (void)
+{
+        g_assert (panel_lockdown.initialized != FALSE);
+
+        return !panel_lockdown.locked_down;
+}
+
 gboolean
 panel_lockdown_get_disable_command_line (void)
 {
