@@ -56,6 +56,7 @@ struct _PanelMenuBarPrivate {
 	AppletInfo  *info;
 	PanelWidget *panel;
 
+	GtkWidget   *image;
 	GtkWidget   *applications_menu;
 	GtkWidget   *applications_item;
 	GtkWidget   *places_item;
@@ -71,6 +72,18 @@ enum {
 
 static void panel_menu_bar_update_text_gravity (PanelMenuBar *menubar);
 
+/* themeable size - "panel-foobar" -- This is only used for the icon of the
+ * Applications item in the menu bar */
+#define PANEL_DEFAULT_MENU_BAR_ICON_SIZE       24
+
+static GtkIconSize panel_menu_bar_icon_size = 0;
+
+static GtkIconSize
+panel_menu_bar_icon_get_size (void)
+{
+	return panel_menu_bar_icon_size;
+}
+
 static gboolean
 panel_menu_bar_reinit_tooltip (GtkWidget    *widget,
 			       PanelMenuBar *menubar)
@@ -131,13 +144,44 @@ panel_menu_bar_setup_tooltip (PanelMenuBar *menubar)
 			  menubar);
 }
 
+static void
+_gtk_label_make_bold (GtkLabel *label)
+{
+        PangoFontDescription *font_desc;
+
+        font_desc = pango_font_description_new ();
+
+        pango_font_description_set_weight (font_desc,
+                                           PANGO_WEIGHT_BOLD);
+
+        /* This will only affect the weight of the font, the rest is
+         * from the current state of the widget, which comes from the
+         * theme or user prefs, since the font desc only has the
+         * weight flag turned on.
+         */
+        gtk_widget_modify_font (GTK_WIDGET (label), font_desc);
+
+        pango_font_description_free (font_desc);
+}
+
 static void
 panel_menu_bar_init (PanelMenuBar *menubar)
 {
-	GtkWidget *image;
+        GtkCssProvider *provider;
 
 	menubar->priv = PANEL_MENU_BAR_GET_PRIVATE (menubar);
 
+        provider = gtk_css_provider_new ();
+        gtk_css_provider_load_from_data (provider,
+                                         "PanelMenuBar {\n"
+                                         " border-width: 0px;\n"
+                                         "}",
+                                         -1, NULL);
+        gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (menubar)),
+                                        GTK_STYLE_PROVIDER (provider),
+                                        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+        g_object_unref (provider);
+
 	menubar->priv->info = NULL;
 
 	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL, TRUE);
@@ -145,10 +189,9 @@ panel_menu_bar_init (PanelMenuBar *menubar)
 	menubar->priv->applications_item = panel_image_menu_item_new ();
 	gtk_menu_item_set_label (GTK_MENU_ITEM (menubar->priv->applications_item),
 				 _("Applications"));
-	image = gtk_image_new_from_icon_name (PANEL_ICON_MAIN_MENU,
-					      panel_menu_bar_icon_get_size ());
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
-				       image);
+	menubar->priv->image = gtk_image_new_from_icon_name (PANEL_ICON_MAIN_MENU,
+							     panel_menu_bar_icon_get_size ());
+        _gtk_label_make_bold (GTK_LABEL (gtk_bin_get_child (GTK_BIN (menubar->priv->applications_item))));
 
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->applications_item),
 				   menubar->priv->applications_menu);
@@ -156,12 +199,14 @@ panel_menu_bar_init (PanelMenuBar *menubar)
 			       menubar->priv->applications_item);
 
 	menubar->priv->places_item = panel_place_menu_item_new (FALSE);
+        _gtk_label_make_bold (GTK_LABEL (gtk_bin_get_child (GTK_BIN (menubar->priv->places_item))));
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->places_item);
 
 	menubar->priv->desktop_item = panel_desktop_menu_item_new (FALSE, TRUE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menubar),
 			       menubar->priv->desktop_item);
+        _gtk_label_make_bold (GTK_LABEL (gtk_bin_get_child (GTK_BIN (menubar->priv->desktop_item))));
 
 	panel_menu_bar_setup_tooltip (menubar);
 
@@ -220,10 +265,12 @@ panel_menu_bar_parent_set (GtkWidget *widget,
 			   GtkWidget *previous_parent)
 {
 	PanelMenuBar *menubar = PANEL_MENU_BAR (widget);
+	GtkWidget    *parent;
 
-	g_assert (!widget->parent || PANEL_IS_WIDGET (widget->parent));
+	parent = gtk_widget_get_parent (widget);
+	g_assert (!parent || PANEL_IS_WIDGET (parent));
 
-	menubar->priv->panel = (PanelWidget *) widget->parent;
+	menubar->priv->panel = (PanelWidget *) parent;
 
 	if (menubar->priv->applications_menu)
 		panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
@@ -242,12 +289,15 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 			      GtkAllocation *allocation)
 {
 	GtkAllocation    old_allocation;
+	GtkAllocation    widget_allocation;
 	PanelBackground *background;
 
-	old_allocation.x      = widget->allocation.x;
-	old_allocation.y      = widget->allocation.y;
-	old_allocation.width  = widget->allocation.width;
-	old_allocation.height = widget->allocation.height;
+	gtk_widget_get_allocation (widget, &widget_allocation);
+
+	old_allocation.x      = widget_allocation.x;
+	old_allocation.y      = widget_allocation.y;
+	old_allocation.width  = widget_allocation.width;
+	old_allocation.height = widget_allocation.height;
 
 	GTK_WIDGET_CLASS (panel_menu_bar_parent_class)->size_allocate (widget, allocation);
 
@@ -266,6 +316,24 @@ panel_menu_bar_size_allocate (GtkWidget     *widget,
 	panel_menu_bar_change_background (PANEL_MENU_BAR (widget));
 }
 
+static void
+panel_menu_bar_style_updated (GtkWidget *widget)
+{
+	PanelMenuBar *menubar = PANEL_MENU_BAR (widget);
+	gboolean      visible;
+
+	GTK_WIDGET_CLASS (panel_menu_bar_parent_class)->style_updated (widget);
+
+	gtk_widget_style_get (widget, "icon-visible", &visible, NULL);
+
+	if (visible)
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
+					       menubar->priv->image);
+	else
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menubar->priv->applications_item),
+					       NULL);
+}
+
 static void
 panel_menu_bar_class_init (PanelMenuBarClass *klass)
 {
@@ -277,6 +345,7 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 
 	widget_class->parent_set = panel_menu_bar_parent_set;
 	widget_class->size_allocate = panel_menu_bar_size_allocate;
+	widget_class->style_updated = panel_menu_bar_style_updated;
 
 	g_type_class_add_private (klass, sizeof (PanelMenuBarPrivate));
 
@@ -290,30 +359,44 @@ panel_menu_bar_class_init (PanelMenuBarClass *klass)
 				   PANEL_ORIENTATION_TOP,
 				   G_PARAM_READWRITE));
 
-	gtk_rc_parse_string (
-		"style \"panel-menubar-style\"\n"
-		"{\n"
-		"  GtkMenuBar::shadow-type = none\n"
-		"  GtkMenuBar::internal-padding = 0\n"
-		"}\n"
-		"class \"PanelMenuBar\" style \"panel-menubar-style\"");
+	gtk_widget_class_install_style_property (
+		widget_class,
+		g_param_spec_boolean ("icon-visible",
+				      "Icon visible",
+				      "Whether the menubar icon is visible",
+				      FALSE,
+				      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+	if (panel_menu_bar_icon_size == 0)
+		panel_menu_bar_icon_size = gtk_icon_size_register ("panel-foobar",
+								   PANEL_DEFAULT_MENU_BAR_ICON_SIZE,
+								   PANEL_DEFAULT_MENU_BAR_ICON_SIZE);
 }
 
 static gboolean
-panel_menu_bar_on_expose (GtkWidget      *widget,
-			  GdkEventExpose *event,
-			  gpointer        data)
+panel_menu_bar_on_draw (GtkWidget *widget,
+			cairo_t   *cr,
+			gpointer   data)
 {
 	PanelMenuBar *menubar = data;
 
-	if (gtk_widget_has_focus (GTK_WIDGET (menubar)))
-		gtk_paint_focus (widget->style,
-				 widget->window, 
-				 GTK_WIDGET_STATE (menubar),
-				 NULL,
-				 widget,
-				 "menubar-applet",
-				 0, 0, -1, -1);
+	if (gtk_widget_has_focus (GTK_WIDGET (menubar))) {
+                GtkStyleContext *context;
+
+                context = gtk_widget_get_style_context (widget);
+                gtk_style_context_save (context);
+                gtk_style_context_set_state (context, gtk_widget_get_state_flags (widget));
+
+                cairo_save (cr);
+		gtk_render_focus (context, cr,
+                                  0, 0,
+                                  gtk_widget_get_allocated_width (widget),
+                                  gtk_widget_get_allocated_height (widget));
+                cairo_restore (cr);
+
+                gtk_style_context_restore (context);
+        }
+
 	return FALSE;
 }
 
@@ -339,28 +422,28 @@ panel_menu_bar_load (PanelWidget *panel,
 		return;
 	}
 
-	panel_applet_add_callback (menubar->priv->info,
-				   "help",
-				   GTK_STOCK_HELP,
-				   _("_Help"),
-				   NULL);
-
 	if (panel_is_program_in_path ("alacarte") ||
 	    panel_is_program_in_path ("gmenu-simple-editor")) {
 		panel_applet_add_callback (menubar->priv->info,
 					   "edit",
 					   NULL,
 					   _("_Edit Menus"),
-					   NULL);
+					   panel_lockdown_get_not_locked_down);
 	}
 
+	panel_applet_add_callback (menubar->priv->info,
+				   "help",
+				   GTK_STOCK_HELP,
+				   _("_Help"),
+				   NULL);
+
 	g_signal_connect_after (menubar, "focus-in-event",
 				G_CALLBACK (gtk_widget_queue_draw), menubar);
 	g_signal_connect_after (menubar, "focus-out-event",
 				G_CALLBACK (gtk_widget_queue_draw), menubar);
-	g_signal_connect_after (menubar, "expose-event",
-				G_CALLBACK (panel_menu_bar_on_expose), menubar);
-	GTK_WIDGET_SET_FLAGS (menubar, GTK_CAN_FOCUS);
+	g_signal_connect_after (menubar, "draw",
+				G_CALLBACK (panel_menu_bar_on_draw), menubar);
+	gtk_widget_set_can_focus (GTK_WIDGET (menubar), TRUE);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
 }
@@ -435,11 +518,15 @@ panel_menu_bar_popup_menu (PanelMenuBar *menubar,
 	 * As that function is private its code is replicated here.
 	 */
 	menu_shell = GTK_MENU_SHELL (menubar);
-	if (!menu_shell->active) {
+	/* FIXMEgpoo: We need either accessors or a workaround to
+	   grab the focus  */
+#if 0
+	if (!menu_shell->GSEAL(active)) {
 		gtk_grab_add (GTK_WIDGET (menu_shell));
-		menu_shell->have_grab = TRUE;
-		menu_shell->active = TRUE;
+		menu_shell->GSEAL(have_grab) = TRUE;
+		menu_shell->GSEAL(active) = TRUE;
 	}
+#endif
 	gtk_menu_shell_select_item (menu_shell,
 				    gtk_menu_get_attach_widget (menu));
 }
@@ -458,7 +545,7 @@ set_item_text_gravity (GtkWidget *item)
 	PangoLayout  *layout;
 	PangoContext *context;
 
-	label = GTK_BIN (item)->child;
+	label = gtk_bin_get_child (GTK_BIN (item));
 	layout = gtk_label_get_layout (GTK_LABEL (label));
 	context = pango_layout_get_context (layout);
 	pango_context_set_base_gravity (context, PANGO_GRAVITY_AUTO);
@@ -480,7 +567,7 @@ set_item_text_angle_and_alignment (GtkWidget *item,
 {
 	GtkWidget *label;
 
-	label = GTK_BIN (item)->child;
+	label = gtk_bin_get_child (GTK_BIN (item));
 
 	gtk_label_set_angle (GTK_LABEL (label), text_angle);
 
