@@ -36,6 +36,7 @@
 #include <libpanel-util/panel-show.h>
 
 #include "applet.h"
+#include "panel-compatibility.h"
 #include "panel-widget.h"
 #include "panel-util.h"
 #include "panel-profile.h"
@@ -64,6 +65,7 @@ typedef enum {
 	APPLICATIONS_MENU,
 #define DEFAULT_MENU      APPLICATIONS_MENU
 	SETTINGS_MENU,
+	GNOMECC_MENU,
 	LAST_MENU
 } MenuPathRoot;
 
@@ -75,7 +77,9 @@ typedef struct {
 
 static MenuPathRootItem root_items [] = {
 	{ APPLICATIONS_MENU, "applications", "applications.menu" },
-	{ SETTINGS_MENU,     "settings",     "settings.menu"     }
+	{ GNOMECC_MENU,      "gnomecc",      "gnomecc.menu"      },
+	/* compatibility with GNOME < 2.91.0 */
+	{ SETTINGS_MENU,     "settings",     NULL                }
 };
 
 struct _PanelMenuButtonPrivate {
@@ -307,11 +311,13 @@ panel_menu_button_parent_set (GtkWidget *widget,
 			      GtkWidget *previous_parent)
 {
 	PanelMenuButton *button = PANEL_MENU_BUTTON (widget);
+	GtkWidget       *parent;
 
-	g_return_if_fail (!widget->parent || PANEL_IS_WIDGET (widget->parent));
+	parent = gtk_widget_get_parent (widget);
+	g_return_if_fail (!parent || PANEL_IS_WIDGET (parent));
 
-	if (widget->parent)
-		button->priv->toplevel = PANEL_WIDGET (widget->parent)->toplevel;
+	if (parent)
+		button->priv->toplevel = PANEL_WIDGET (parent)->toplevel;
 	else
 		button->priv->toplevel = NULL;
 
@@ -336,7 +342,7 @@ panel_menu_button_drag_data_get (GtkWidget        *widget,
 	drag_data = g_strdup_printf ("MENU:%d", panel_find_applet_index (widget));
 
 	gtk_selection_data_set (
-		selection_data, selection_data->target,
+		selection_data, gtk_selection_data_get_target (selection_data),
 		8, (guchar *) drag_data, strlen (drag_data));
 
 	g_free (drag_data);
@@ -347,7 +353,9 @@ panel_menu_button_menu_deactivated (PanelMenuButton *button)
 {
 	panel_toplevel_pop_autohide_disabler (button->priv->toplevel);
 
-	GTK_BUTTON (button)->in_button = FALSE;
+	gtk_widget_unset_state_flags (GTK_WIDGET (button),
+				      GTK_STATE_FLAG_PRELIGHT);
+
 	button_widget_set_ignore_leave (BUTTON_WIDGET (button), FALSE);
 }
 
@@ -406,8 +414,17 @@ panel_menu_button_create_menu (PanelMenuButton *button)
 static void
 panel_menu_button_recreate_menu (PanelMenuButton *button)
 {
-	if (button->priv->menu)
+	if (button->priv->menu) {
+		if (gtk_widget_get_visible (button->priv->menu))
+			gtk_menu_shell_deactivate (GTK_MENU_SHELL (button->priv->menu));
+
+		g_signal_handlers_disconnect_by_func (button->priv->menu,
+						      G_CALLBACK (panel_menu_button_menu_deactivated),
+						      button);
+
 		gtk_widget_destroy (button->priv->menu);
+	}
+
 	button->priv->menu = NULL;
 }
 
@@ -660,12 +677,11 @@ panel_menu_button_load (const char  *menu_path,
 
 	button->priv->applet_id = g_strdup (info->id);
 
-	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"), NULL);
-
         if (panel_is_program_in_path ("alacarte") ||
 	    panel_is_program_in_path ("gmenu-simple-editor"))
 		panel_applet_add_callback (info, "edit", NULL,
-					   _("_Edit Menus"), NULL);
+					   _("_Edit Menus"),
+					   panel_lockdown_get_not_locked_down);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
@@ -868,19 +884,30 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 				   gboolean     exactpos,
 				   const char  *id)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *menu_path;
-	char        *custom_icon;
-	char        *tooltip;
-	gboolean     use_menu_path;
-	gboolean     use_custom_icon;
+	GConfClient  *client;
+	char         *scheme;
+	MenuPathRoot  root;
+	const char   *key;
+	char         *menu_path;
+	char         *custom_icon;
+	char         *tooltip;
+	gboolean      use_menu_path;
+	gboolean      use_custom_icon;
 
 	client  = panel_gconf_get_client ();
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
 	menu_path = gconf_client_get_string (client, key, NULL);
 
+	scheme = g_strndup (menu_path, strcspn (menu_path, ":"));
+	root = panel_menu_scheme_to_path_root (scheme);
+	g_free (scheme);
+	if (root == SETTINGS_MENU) {
+		panel_compatiblity_migrate_settings_menu_button (client, id);
+		g_free (menu_path);
+		return;
+       }
+
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "custom_icon");
 	custom_icon = gconf_client_get_string (client, key, NULL);
 
@@ -1006,7 +1033,7 @@ panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
 		};
 		char *icon;
 
-		GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
+		gtk_widget_set_has_window (GTK_WIDGET (button), TRUE);
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
@@ -1018,7 +1045,7 @@ panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
 			g_free (icon);
 		}
 
-		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
+		gtk_widget_set_has_window (GTK_WIDGET (button), FALSE);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
 }
@@ -1043,7 +1070,11 @@ panel_menu_button_accessible_get_n_children (AtkObject *obj)
 {
 	g_return_val_if_fail (PANEL_IS_MENU_BUTTON_ACCESSIBLE (obj), 0);
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+	return gtk_accessible_get_widget (GTK_ACCESSIBLE (obj)) ? 1 : 0;
+#else
 	return GTK_ACCESSIBLE (obj)->widget ? 1 : 0;
+#endif
 }
 
 static AtkObject *
@@ -1058,7 +1089,11 @@ panel_menu_button_accessible_ref_child (AtkObject *obj,
 	if (index != 0)
 		return NULL;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+	if (!(button = PANEL_MENU_BUTTON (gtk_accessible_get_widget (GTK_ACCESSIBLE (obj)))))
+#else
 	if (!(button = PANEL_MENU_BUTTON (GTK_ACCESSIBLE (obj)->widget)))
+#endif
 		return NULL;
 
 	if (!(menu = panel_menu_button_create_menu (button)))
